import os
from datetime import datetime

import pandas as pd
from pymongo import MongoClient
from pymongo.collection import InsertManyResult
from statsmodels.tsa.seasonal import seasonal_decompose

from uw_file_processor_api.workflows.steps.commons.scripts import generate_arima_forecast, preparing_data


class ItemProcessorService:
    sales_collection: str = 'sales'
    item_processing_status_collection: str = 'item_processing_status'
    prediction_connection: str = 'predictions'

    tenant: str
    keys: list[str] = []

    mongo_db_name: str
    mongo_client: MongoClient
    mongo_client_db = None

    seasonal_decompose_period: int

    def __init__(self, tenant: str, keys: list[str], seasonal_decompose_period: int = 12):
        if tenant in ('', None):
            raise Exception('tenant cannot be empty')

        if not isinstance(keys, list):
            raise Exception('keys must be a list of strings')

        if not keys:
            raise Exception('keys cannot be empty')

        self.tenant = tenant
        self.keys = keys
        self.seasonal_decompose_period = seasonal_decompose_period

        # mongo client
        mongo_connection_string = os.getenv('MONGO_CONNECTION_STRING',
                                            'mongodb://localhost:27017/?directConnection=true')

        self.mongo_client = MongoClient(mongo_connection_string)

        self.mongo_db_name = f'client-{tenant}'
        self.mongo_client_db = self.mongo_client[self.mongo_db_name]

    def process_items(self):
        items = self.get_to_process_items_by_keys()

        for index, item in enumerate(items):
            filters = {}
            for key in self.keys:
                filters[key] = item[key]

            exception_message = None
            try:
                df_base = self.get_item_data_to_processing(filters)
                df_prepared = self.preparing_data_to_prediction(df_base)
                df_forecast = self.prediction_with_arima_model(filters, df_prepared)
                df_seasonality = self.process_seasonal_decompose(filters, df_prepared)
                saved_ids = self.save_result_in_db(filters, df_prepared, df_forecast, df_seasonality)

                print(f'saved ids data: {len(saved_ids.inserted_ids)}')
            except Exception as e:
                print(f'error process_item: {e}')
                exception_message = str(e)

            status = 'Succeeded'
            if exception_message:
                status = 'Failed'

            self.process_item_save_status(filters, status, exception_message)

    def process_item_save_status(self, filters: dict, status: str, exception: str = None):
        try:
            mongo_collection_process_all_by_keys = self.mongo_client_db[self.item_processing_status_collection]

            body_to_save = {
                'status': status,
                'created_at': datetime.now(),
            }

            if exception not in ['', None]:
                body_to_save['exception'] = exception

            updated_items = mongo_collection_process_all_by_keys.update_one(
                filter=filters,
                update={'$set': body_to_save},
                upsert=True
            )

            print(f'updated items count: {updated_items.modified_count}')

        except Exception as e:
            print(f'process_item_save_status error saving status: {e}')
            raise e

    def save_result_in_db(self, filters: dict, prepared_df: pd.DataFrame, arima_df: pd.DataFrame,
                          seasonal_df: pd.DataFrame) -> InsertManyResult:
        try:
            keys = []
            for key, value in filters.items():
                prepared_df[key] = value
                keys.append(key)

            grouper = [pd.Grouper(freq='M', key='date')] + self.keys

            prepared_df = prepared_df.reset_index().groupby(grouper).last().sort_index()
            arima_df = arima_df.reset_index().groupby(grouper).last().sort_index()
            seasonal_df = seasonal_df.reset_index().groupby(grouper).last().sort_index()

            df_final = pd.concat([prepared_df, arima_df, seasonal_df])

            mongo_collection = self.mongo_client_db[self.prediction_connection]

            df_final = df_final.fillna(0.0)

            return mongo_collection.insert_many(df_final.reset_index().to_dict('records'))
        except Exception as e:
            print(f'error save_result_in_db: {e}')
            raise e

    def process_seasonal_decompose(self, filters: dict, df: pd.DataFrame) -> pd.DataFrame:
        try:
            data_to_seasonal_decompose = df.reset_index().groupby([pd.Grouper(freq='M', key='date')]).agg(
                {'quantity': 'sum'}).sort_index()

            result = seasonal_decompose(data_to_seasonal_decompose, model='additive',
                                        period=self.seasonal_decompose_period)

            trend = pd.DataFrame(result.trend)
            seasonal = pd.DataFrame(result.seasonal)
            residuals = pd.DataFrame(result.resid)

            df_result = pd.concat([trend, seasonal, residuals]).reset_index().groupby(
                [pd.Grouper(freq='M', key='date')]).agg(
                {
                    'trend': 'last',
                    'seasonal': 'last',
                    'residuals': 'last'
                }
            ).sort_index()

            for key, value in filters.items():
                df_result[key] = value

            return df_result
        except Exception as e:
            print(f'error process_seasonal_decompose: {e}')
            raise e

    def prediction_with_arima_model(self, filters: dict, df: pd.DataFrame) -> pd.DataFrame:
        try:
            df_forecast = generate_arima_forecast(df)

            if df_forecast.empty:
                raise Exception('No forecast generated')

            for key, value in filters.items():
                df_forecast[key] = value

            return df_forecast
        except Exception as e:
            print(f'error prediction_with_arima_model: {e}')
            raise e

    def preparing_data_to_prediction(self, df: pd.DataFrame) -> pd.DataFrame:
        try:
            df_prepared = preparing_data(df)

            if df_prepared.empty:
                raise Exception('No data found')

            print('dataframe head:\n', df_prepared.head())

            return df
        except Exception as e:
            print(f'error preparing_data_to_prediction: {e}')
            raise e

    def get_item_data_to_processing(self, filters: dict) -> pd.DataFrame:
        try:
            mongo_collection_sales = self.mongo_client_db[self.sales_collection]

            mongo_collection_process_all_by_keys = self.mongo_client_db[self.item_processing_status_collection]

            item_status = mongo_collection_process_all_by_keys.find_one(filters)

            if item_status is not None:
                if item_status['status'] != 'Pending':
                    raise Exception(f'item status is {item_status["status"]}')

            documents = []
            for document in mongo_collection_sales.find(filters):
                documents.append(document)

            df = pd.DataFrame(documents)

            if df.empty:
                raise Exception('No data found')

            print('dataframe head:\n', df.head())

            df = df.drop(columns=['_id'])

            if df.empty:
                raise Exception('No data found')

            return df

        except Exception as e:
            print(f'error get_item_data_to_processing: {e}')
            raise e

    def get_to_process_items_by_keys(self) -> list[dict]:
        pipeline_group = {
            '$group': {
                '_id': {},
            }
        }

        pipeline_project = {
            '$project': {
                '_id': 0
            }
        }

        for key in self.keys:
            pipeline_group['$group']['_id'][key] = f'${key}'
            pipeline_project['$project'][key] = f'$_id.{key}'

        pipeline = [pipeline_group, pipeline_project]

        mongo_db = self.mongo_client[self.mongo_db_name]
        mongo_collection_sales = mongo_db[self.sales_collection]

        result: list[dict] = []
        for doc in mongo_collection_sales.aggregate(pipeline):
            result.append(doc)

        return result
