import json

import pandas as pd
from pymongo import MongoClient
from pymongo.results import InsertManyResult
from sklearn.preprocessing import StandardScaler
from statsmodels.tsa.arima.model import ARIMA


def get_and_validate_body(body: str) -> dict:
    body = json.loads(body)

    if 'tenant' not in body:
        raise Exception('No tenant found in body')

    if 'filters' not in body:
        raise Exception('No filters found in body')

    return body


def generate_arima_forecast(df: pd.DataFrame) -> pd.DataFrame:
    df = df.set_index('date')

    # Scale data
    model_scale = StandardScaler()
    model_scale.fit(df)
    scaled_values = model_scale.transform(df)

    scaled_values_df = pd.DataFrame(data=scaled_values.reshape(-1),
                                    index=df.index.to_numpy(),
                                    columns=['quantity'])
    scaled_values_df.index.set_names('date', inplace=True)

    # Create model
    scaled_values_df.index = pd.DatetimeIndex(scaled_values_df.index.values,
                                              freq=scaled_values_df.index.inferred_freq)
    model = ARIMA(scaled_values_df, order=(1, 1, 1))
    model_fit = model.fit()

    # Train
    scaled_train_result = model_fit.fittedvalues.to_numpy().reshape(-1, 1)
    train_data = model_scale.inverse_transform(scaled_train_result)
    train = pd.DataFrame(data={
        'train': train_data.reshape(-1)
    }, index=model_fit.fittedvalues.index)
    train.index.set_names('date', inplace=True)

    # Forecast
    scaled_forecast_result = model_fit.forecast()
    scaled_forecast = scaled_forecast_result.to_numpy().reshape(-1, 1)
    forecast = model_scale.inverse_transform(scaled_forecast)
    forecast = pd.DataFrame(data={
        'forecast': forecast.reshape(-1)
    }, index=scaled_forecast_result.index)
    forecast.index.set_names('date', inplace=True)
    forecast = forecast.abs()

    # Join train and forecast
    df_final = pd.concat([train, forecast])
    df_final = df_final.reset_index().groupby(['date']).last().sort_index()
    df_final = df_final.fillna(0)

    return df_final


def preparing_data(data: pd.DataFrame) -> pd.DataFrame:
    if data.empty:
        raise Exception('Empty dataframe')

    data = data.reset_index()

    if '_id' in data.columns:
        data = data.drop(columns=['_id'])

    if 'date' not in data.columns:
        raise Exception('date column not found')

    if 'quantity' not in data.columns:
        raise Exception('quantity column not found')

    keys = data.drop(columns=['index', 'date', 'quantity']).columns.tolist()

    grouper_index = [pd.Grouper(freq='M', key='date')] + keys

    base = data.groupby(grouper_index).agg({'quantity': 'sum'}).sort_index()

    base = base.reset_index()

    import warnings
    warnings.filterwarnings("ignore")

    # prepare data
    df = base.drop(columns=keys)
    df = df.set_index('date')
    index_ = pd.date_range(df.index.min(), df.index.max(), freq='M')
    df = df.reindex(index_, fill_value=0).sort_index()
    df = df.fillna(0.0)

    if df.size < 10:
        raise Exception(f'Not enough data to generate forecast, total months: {df.size}')

    # verify if exist sales in the last 3 months
    last_values = df.tail(3)
    last_values = last_values[last_values['quantity'] > 0.0]
    if last_values.size < 3:
        raise Exception('No way out in the last 3 months')

    df.index.set_names('date', inplace=True)

    return df.reset_index()


def get_item_data_from_local_file(file_path: str) -> pd.DataFrame:
    df = pd.read_parquet(file_path)

    return df


def save_data_in_local(df: pd.DataFrame, path_to_save: str):
    df.to_parquet(path_to_save, engine='pyarrow', index=False)


def save_in_mongo_client(mongo_client: MongoClient, tenant: str, collection: str, data: list[dict]) -> InsertManyResult:
    tenant_db_name = f'client-{tenant}'
    mongo_db = mongo_client[tenant_db_name]
    mongo_collection = mongo_db[collection]

    return mongo_collection.insert_many(data)


def get_sales_data_from_mongo_client(mongo_client: MongoClient, collection: str, tenant: str,
                                     filters: dict) -> pd.DataFrame:
    tenant_db_name = f'client-{tenant}'
    mongo_db = mongo_client[tenant_db_name]
    mongo_collection_sales = mongo_db[collection]

    documents = []
    for document in mongo_collection_sales.find(filters):
        documents.append(document)

    return pd.DataFrame(documents)
