import argparse
import json
import os

import pandas as pd
from confluent_kafka import Producer
from pymongo import MongoClient

SALES_COLLECTION = 'sales'
PROCESS_ALL_BY_KEYS_COLLECTION = 'process_all_by_keys'

KAFKA_TOPIC = 'process-item'


def main():
    parser = argparse.ArgumentParser(description="Process args.")

    parser.add_argument('--body', type=str, help='body of the request', required=True)

    args = parser.parse_args()

    try:
        body = json.loads(args.body)

        if 'tenant' not in body:
            raise Exception('No tenant found in body')

        if 'keys' not in body:
            raise Exception('No tenant found in body')

        # verify if keys is a list of strings
        if not isinstance(body['keys'], list):
            raise Exception('keys must be a list of strings')

        # verify if keys is not empty
        if not body['keys']:
            raise Exception('keys cannot be empty')

        pipeline_group = {
            '$group': {
                '_id': {},
            }
        }

        pipeline_project = {
            '$project': {
                '_id': 0
            }
        }

        for key in body['keys']:
            pipeline_group['$group']['_id'][key] = f'${key}'
            pipeline_project['$project'][key] = f'$_id.{key}'

        pipeline = [pipeline_group, pipeline_project]

        mongo_connection_string = os.getenv('MONGO_CONNECTION_STRING',
                                            'mongodb://localhost:27017/?directConnection=true')

        mongo_client = MongoClient(mongo_connection_string)

        mongo_db_name = f'client-{body["tenant"]}'
        mongo_db = mongo_client[mongo_db_name]
        mongo_collection_sales = mongo_db[SALES_COLLECTION]

        result = []
        for doc in mongo_collection_sales.aggregate(pipeline):
            result.append(doc)

        # kafka producer config
        kafka_servers = os.getenv('KAFKA_BOOTSTRAP_SERVERS', 'kafka.kafka:9092')

        kafka_conf = {
            'bootstrap.servers': kafka_servers,
        }

        kafka_producer = Producer(kafka_conf)

        mongo_collection_process_all_by_keys = mongo_db[PROCESS_ALL_BY_KEYS_COLLECTION]

        for doc in result:
            kafka_body = {
                'tenant': body['tenant'],
                'filters': doc
            }

            kafka_producer.produce(KAFKA_TOPIC, json.dumps(kafka_body).encode('utf-8'))
            doc['status'] = 'Pending'
            doc['created_at'] = pd.Timestamp.now()
            mongo_collection_process_all_by_keys.insert_one(doc)

        kafka_producer.flush()

        print(f'sent {len(result)} messages to kafka')

    except Exception as e:
        print(f'error processing all by keys: {e}')
        raise e

    print('File downloaded successfully')


if __name__ == "__main__":
    main()
