import argparse
import json

import pandas as pd
from statsmodels.tsa.seasonal import seasonal_decompose

from uw_file_processor_api.workflows.steps.commons.scripts import get_item_data_from_local_file, \
    save_data_in_local


def main():
    parser = argparse.ArgumentParser(description="Process args.")

    parser.add_argument('--body', type=str, help='body of the request', required=True)
    parser.add_argument('--period', type=int, help='period of seasonal decomposition', default=6)
    parser.add_argument('--prepared-files-path', type=str, help='path to prepared files',
                        default='/data/prepared_files.parquet')
    parser.add_argument('--seasonal-decompose-path', type=str, help='path to seasonal decompose',
                        default='/data/seasonal_decompose.parquet')

    args = parser.parse_args()

    try:
        df = get_item_data_from_local_file(args.prepared_files_path)

        if df.empty:
            raise Exception('No data found')

        data_to_seasonal_decompose = df.reset_index().groupby([pd.Grouper(freq='M', key='date')]).agg(
            {'quantity': 'sum'}).sort_index()

        result = seasonal_decompose(data_to_seasonal_decompose, model='additive',
                                    period=args.period)

        trend = pd.DataFrame(result.trend)
        seasonal = pd.DataFrame(result.seasonal)
        resid = pd.DataFrame(result.resid)

        df_result = pd.concat([trend, seasonal, resid]).reset_index().groupby([pd.Grouper(freq='M', key='date')]).agg(
            {
                'trend': 'last',
                'seasonal': 'last',
                'resid': 'last'
            }
        ).sort_index()

        body = json.loads(args.body)

        if 'tenant' not in body:
            raise Exception('No tenant found in body')

        if 'filters' not in body:
            raise Exception('No filters found in body')

        for key, value in body['filters'].items():
            df_result[key] = value

        save_data_in_local(df_result.reset_index(), args.seasonal_decompose_path)

        print('dataframe head:\n', df_result.head())

    except Exception as e:
        print(f'Error processing file: {e}')
        raise e


if __name__ == "__main__":
    main()
