import argparse
import os

import pandas as pd
from pymongo import MongoClient

from uw_file_processor_api.workflows.steps.commons.scripts import get_item_data_from_local_file, \
    get_and_validate_body

PREDICTION_COLLECTION = 'prediction'


def main():
    parser = argparse.ArgumentParser(description="Process args.")

    parser.add_argument('--body', type=str, help='body of the request', required=True)
    parser.add_argument('--prepared-files-path', type=str, help='path to prepared files',
                        default='/data/prepared_files.parquet')
    parser.add_argument('--arima-forecast-path', type=str, help='arima file path to process',
                        default='/data/arima_forecast.parquet')
    parser.add_argument('--seasonal-decompose-path', type=str, help='seasonal file path to process',
                        default='/data/seasonal_decompose.parquet')

    args = parser.parse_args()

    try:

        prepared_df = get_item_data_from_local_file(args.prepared_files_path)
        arima_df = get_item_data_from_local_file(args.arima_forecast_path)
        seasonal_df = get_item_data_from_local_file(args.seasonal_decompose_path)

        if prepared_df.empty:
            raise Exception('prepared_df not found')

        if arima_df.empty:
            raise Exception('arima_df not found')

        if seasonal_df.empty:
            raise Exception('seasonal_df not found')

        body = get_and_validate_body(args.body)

        keys = []
        for key, value in body['filters'].items():
            prepared_df[key] = value
            keys.append(key)

        grouper = [pd.Grouper(freq='M', key='date')] + keys

        prepared_df = prepared_df.reset_index().groupby(grouper).last().sort_index()
        arima_df = arima_df.reset_index().groupby(grouper).last().sort_index()
        seasonal_df = seasonal_df.reset_index().groupby(grouper).last().sort_index()

        df_final = pd.concat([prepared_df, arima_df, seasonal_df])

        print('dataframe head:\n', df_final.head())

        mongo_connection_string = os.getenv('MONGO_CONNECTION_STRING',
                                            'mongodb://localhost:27017/?directConnection=true')

        mongo_client = MongoClient(mongo_connection_string)

        tenant_db_name = f'client-{body["tenant"]}'
        mongo_db = mongo_client[tenant_db_name]
        mongo_collection = mongo_db[PREDICTION_COLLECTION]

        df_final = df_final.fillna(0.0)

        ids = mongo_collection.insert_many(df_final.reset_index().to_dict('records'))

        print('total inserted: ', len(ids.inserted_ids))

    except Exception as e:
        print(f'error to save data in mongo: {e}')
        raise e


if __name__ == "__main__":
    main()
